"""
A module containing LUTs for primitive elements in GF(5^m).

Sage:
    order = 5
    degree = 1
    F.<x> = GF(order**degree, repr="int")
    print(f"PRIMITIVE_ELEMENTS_{order}_{degree} = [")
    for e in range(1, F.order()):
        e = F.fetch_int(e)
        if e.multiplicative_order() == F.order() - 1:
            c = e.polynomial().coefficients(sparse=False)[::-1]
            print(f"    {c},")
    print("]\n")
"""

# LUT items are poly coefficients in degree-descending order

PRIMITIVE_ELEMENTS_5_2 = [
    [1, 0],
    [1, 4],
    [2, 0],
    [2, 3],
    [3, 0],
    [3, 2],
    [4, 0],
    [4, 1],
]

PRIMITIVE_ELEMENTS_5_3 = [
    [1, 0],
    [1, 3],
    [1, 4],
    [2, 2],
    [2, 4],
    [3, 1],
    [3, 3],
    [4, 0],
    [4, 1],
    [4, 2],
    [1, 0, 1],
    [1, 0, 4],
    [1, 1, 0],
    [1, 1, 1],
    [1, 1, 2],
    [1, 1, 3],
    [1, 2, 0],
    [1, 2, 3],
    [1, 3, 3],
    [1, 4, 3],
    [2, 0, 0],
    [2, 0, 1],
    [2, 0, 4],
    [2, 1, 0],
    [2, 1, 2],
    [2, 1, 3],
    [2, 1, 4],
    [2, 2, 3],
    [2, 3, 0],
    [2, 3, 2],
    [2, 3, 3],
    [2, 3, 4],
    [2, 4, 2],
    [2, 4, 3],
    [2, 4, 4],
    [3, 0, 0],
    [3, 0, 1],
    [3, 0, 4],
    [3, 1, 1],
    [3, 1, 2],
    [3, 1, 3],
    [3, 2, 0],
    [3, 2, 1],
    [3, 2, 2],
    [3, 2, 3],
    [3, 3, 2],
    [3, 4, 0],
    [3, 4, 1],
    [3, 4, 2],
    [3, 4, 3],
    [4, 0, 1],
    [4, 0, 4],
    [4, 1, 2],
    [4, 2, 2],
    [4, 3, 0],
    [4, 3, 2],
    [4, 4, 0],
    [4, 4, 2],
    [4, 4, 3],
    [4, 4, 4],
]

PRIMITIVE_ELEMENTS_5_4 = [
    [1, 0],
    [1, 1],
    [1, 3],
    [2, 0],
    [2, 1],
    [2, 2],
    [3, 0],
    [3, 3],
    [3, 4],
    [4, 0],
    [4, 2],
    [4, 4],
    [1, 0, 1],
    [1, 0, 4],
    [1, 1, 1],
    [1, 2, 0],
    [1, 2, 4],
    [1, 3, 3],
    [1, 3, 4],
    [1, 4, 0],
    [2, 0, 2],
    [2, 0, 3],
    [2, 1, 1],
    [2, 1, 3],
    [2, 2, 2],
    [2, 3, 0],
    [2, 4, 0],
    [2, 4, 3],
    [3, 0, 2],
    [3, 0, 3],
    [3, 1, 0],
    [3, 1, 2],
    [3, 2, 0],
    [3, 3, 3],
    [3, 4, 2],
    [3, 4, 4],
    [4, 0, 1],
    [4, 0, 4],
    [4, 1, 0],
    [4, 2, 1],
    [4, 2, 2],
    [4, 3, 0],
    [4, 3, 1],
    [4, 4, 4],
    [1, 0, 0, 3],
    [1, 0, 0, 4],
    [1, 0, 1, 1],
    [1, 0, 1, 2],
    [1, 0, 3, 0],
    [1, 0, 3, 4],
    [1, 0, 4, 1],
    [1, 0, 4, 4],
    [1, 1, 0, 0],
    [1, 1, 0, 1],
    [1, 1, 0, 3],
    [1, 1, 2, 0],
    [1, 1, 2, 1],
    [1, 1, 2, 2],
    [1, 1, 3, 0],
    [1, 1, 3, 1],
    [1, 1, 3, 3],
    [1, 2, 0, 2],
    [1, 2, 0, 4],
    [1, 2, 1, 0],
    [1, 2, 1, 1],
    [1, 2, 1, 3],
    [1, 2, 1, 4],
    [1, 2, 2, 3],
    [1, 2, 3, 0],
    [1, 2, 3, 3],
    [1, 3, 1, 0],
    [1, 3, 1, 2],
    [1, 3, 4, 3],
    [1, 4, 0, 3],
    [1, 4, 0, 4],
    [1, 4, 1, 0],
    [1, 4, 1, 4],
    [1, 4, 3, 0],
    [1, 4, 3, 1],
    [1, 4, 4, 1],
    [1, 4, 4, 4],
    [2, 0, 0, 1],
    [2, 0, 0, 3],
    [2, 0, 1, 0],
    [2, 0, 1, 3],
    [2, 0, 2, 2],
    [2, 0, 2, 4],
    [2, 0, 3, 2],
    [2, 0, 3, 3],
    [2, 1, 2, 0],
    [2, 1, 2, 4],
    [2, 1, 3, 1],
    [2, 2, 0, 0],
    [2, 2, 0, 1],
    [2, 2, 0, 2],
    [2, 2, 1, 0],
    [2, 2, 1, 1],
    [2, 2, 1, 2],
    [2, 2, 4, 0],
    [2, 2, 4, 2],
    [2, 2, 4, 4],
    [2, 3, 0, 1],
    [2, 3, 0, 3],
    [2, 3, 1, 0],
    [2, 3, 1, 2],
    [2, 3, 2, 0],
    [2, 3, 2, 3],
    [2, 3, 3, 2],
    [2, 3, 3, 3],
    [2, 4, 0, 3],
    [2, 4, 0, 4],
    [2, 4, 1, 0],
    [2, 4, 1, 1],
    [2, 4, 2, 0],
    [2, 4, 2, 1],
    [2, 4, 2, 2],
    [2, 4, 2, 3],
    [2, 4, 4, 1],
    [3, 0, 0, 2],
    [3, 0, 0, 4],
    [3, 0, 2, 2],
    [3, 0, 2, 3],
    [3, 0, 3, 1],
    [3, 0, 3, 3],
    [3, 0, 4, 0],
    [3, 0, 4, 2],
    [3, 1, 0, 1],
    [3, 1, 0, 2],
    [3, 1, 1, 4],
    [3, 1, 3, 0],
    [3, 1, 3, 2],
    [3, 1, 3, 3],
    [3, 1, 3, 4],
    [3, 1, 4, 0],
    [3, 1, 4, 4],
    [3, 2, 0, 2],
    [3, 2, 0, 4],
    [3, 2, 2, 2],
    [3, 2, 2, 3],
    [3, 2, 3, 0],
    [3, 2, 3, 2],
    [3, 2, 4, 0],
    [3, 2, 4, 3],
    [3, 3, 0, 0],
    [3, 3, 0, 3],
    [3, 3, 0, 4],
    [3, 3, 1, 0],
    [3, 3, 1, 1],
    [3, 3, 1, 3],
    [3, 3, 4, 0],
    [3, 3, 4, 3],
    [3, 3, 4, 4],
    [3, 4, 2, 4],
    [3, 4, 3, 0],
    [3, 4, 3, 1],
    [4, 0, 0, 1],
    [4, 0, 0, 2],
    [4, 0, 1, 1],
    [4, 0, 1, 4],
    [4, 0, 2, 0],
    [4, 0, 2, 1],
    [4, 0, 4, 3],
    [4, 0, 4, 4],
    [4, 1, 0, 1],
    [4, 1, 0, 2],
    [4, 1, 1, 1],
    [4, 1, 1, 4],
    [4, 1, 2, 0],
    [4, 1, 2, 4],
    [4, 1, 4, 0],
    [4, 1, 4, 1],
    [4, 2, 1, 2],
    [4, 2, 4, 0],
    [4, 2, 4, 3],
    [4, 3, 0, 1],
    [4, 3, 0, 3],
    [4, 3, 2, 0],
    [4, 3, 2, 2],
    [4, 3, 3, 2],
    [4, 3, 4, 0],
    [4, 3, 4, 1],
    [4, 3, 4, 2],
    [4, 3, 4, 4],
    [4, 4, 0, 0],
    [4, 4, 0, 2],
    [4, 4, 0, 4],
    [4, 4, 2, 0],
    [4, 4, 2, 2],
    [4, 4, 2, 4],
    [4, 4, 3, 0],
    [4, 4, 3, 3],
    [4, 4, 3, 4],
]
